# General GUI related functions and structures
use strict;
use warnings;
use Gtk2 -init;

my $builder = Gtk2::Builder->new;
$builder->add_from_file("$Bin/share/remotebox/remotebox.xml");
$builder->connect_signals();

our %gui = (textbufferDescription => $builder->get_object('textbufferDescription'),
            appname               => $builder->get_object('aboutdialog')->get_program_name(),
            appver                => $builder->get_object('aboutdialog')->get_version(),
            websn                 => undef);

# Fill %gui so we can reference them easily.
foreach ($builder->get_objects) {
    my $id;
    eval{ $id = $_->Gtk2::Buildable::get_name; }; # DONT USE $_->get_name()
    $gui{$id} = $_ if ($id);
}

# We have to register signals manually, which we want to block at some point because we need the sigids
# as blocking any other way is not supported in perl-gtk (other blocking funcs map to null funcs)
our %signal = (fam          => $gui{comboboxNewOSFam}->signal_connect(changed => \&tgr_newgen_osfam, $gui{comboboxNewOSVer}),
               ver          => $gui{comboboxNewOSVer}->signal_connect(changed => \&tgr_newgen_osver, $gui{comboboxNewOSFam}),
               famedit      => $gui{comboboxEditGenOSFam}->signal_connect(changed => \&tgr_editgen_osfam, $gui{comboboxEditGenOSVer}),
               veredit      => $gui{comboboxEditGenOSVer}->signal_connect(changed => \&tgr_editgen_osver, $gui{comboboxEditGenOSFam}),
               audiodrv     => $gui{comboboxEditAudioDriver}->signal_connect(changed => \&tgr_editaudio_driver),
               audioctr     => $gui{comboboxEditAudioCtr}->signal_connect(changed => \&tgr_editaudio_ctr),
               netadaptype1 => $gui{comboboxEditNetType1}->signal_connect(changed => \&tgr_editnet_adapter_type, $gui{checkbuttonEditNetEnable1}),
               netadaptype2 => $gui{comboboxEditNetType2}->signal_connect(changed => \&tgr_editnet_adapter_type, $gui{checkbuttonEditNetEnable2}),
               netadaptype3 => $gui{comboboxEditNetType3}->signal_connect(changed => \&tgr_editnet_adapter_type, $gui{checkbuttonEditNetEnable3}),
               netadaptype4 => $gui{comboboxEditNetType4}->signal_connect(changed => \&tgr_editnet_adapter_type, $gui{checkbuttonEditNetEnable4}),
               netattach1   => $gui{comboboxEditNetAttach1}->signal_connect(changed => \&tgr_editnet_attach, $gui{checkbuttonEditNetEnable1}),
               netattach2   => $gui{comboboxEditNetAttach2}->signal_connect(changed => \&tgr_editnet_attach, $gui{checkbuttonEditNetEnable2}),
               netattach3   => $gui{comboboxEditNetAttach3}->signal_connect(changed => \&tgr_editnet_attach, $gui{checkbuttonEditNetEnable3}),
               netattach4   => $gui{comboboxEditNetAttach4}->signal_connect(changed => \&tgr_editnet_attach, $gui{checkbuttonEditNetEnable4}),
               netname1     => $gui{comboboxEditNetName1}->signal_connect(changed => \&tgr_editnet_name, $gui{checkbuttonEditNetEnable1}),
               netname2     => $gui{comboboxEditNetName2}->signal_connect(changed => \&tgr_editnet_name, $gui{checkbuttonEditNetEnable2}),
               netname3     => $gui{comboboxEditNetName3}->signal_connect(changed => \&tgr_editnet_name, $gui{checkbuttonEditNetEnable3}),
               netname4     => $gui{comboboxEditNetName4}->signal_connect(changed => \&tgr_editnet_name, $gui{checkbuttonEditNetEnable4}),
               stortype     => $gui{comboboxEditStorCtrType}->signal_connect(changed => \&tgr_editstor_ctrtype, $gui{treeviewEditStor}));

# Sensivity groups for enabling/disabling different parts of the gui
our %sensgrp = (connect     => [$gui{menuitemNew},
                                $gui{menuitemVMM},
                                $gui{menuitemServerInfo},
                                $gui{toolbuttonNew},
                                $gui{toolbuttonRefresh}],
                running     => [$gui{menuitemStop},
                                $gui{menuitemReset},
                                $gui{menuitemRemoteDisplay},
                                $gui{menuitemTGStop},
                                $gui{menuitemTGReset},
                                $gui{menuitemTGRemoteDisplay},
                                $gui{menuitemTGKeyboard},
                                $gui{toolbuttonStop},
                                $gui{toolbuttonReset},
                                $gui{toolbuttonRemoteDisplay},
                                $gui{buttonShowDetails},
                                $gui{buttonShowSnapshots}],
                saved       => [$gui{menuitemStart},
                                $gui{menuitemTGStart},
                                $gui{toolbuttonStart},
                                $gui{buttonShowDetails},
                                $gui{buttonShowSnapshots}],
                poweroff    => [$gui{menuitemStart},
                                $gui{menuitemSettings},
                                $gui{menuitemDelete},
                                $gui{menuitemTGStart},
                                $gui{menuitemTGSettings},
                                $gui{menuitemTGDelete},
                                $gui{toolbuttonStart},
                                $gui{toolbuttonSettings},
                                $gui{buttonSaveDescription},
                                $gui{buttonShowDetails},
                                $gui{buttonShowSnapshots}],
                unselected  => [$gui{menuitemStart},
                                $gui{menuitemStop},
                                $gui{menuitemSettings},
                                $gui{menuitemReset},
                                $gui{menuitemDelete},
                                $gui{menuitemRemoteDisplay},
                                $gui{menuitemTGStart},
                                $gui{menuitemTGStop},
                                $gui{menuitemTGSettings},
                                $gui{menuitemTGDelete},
                                $gui{menuitemTGReset},
                                $gui{menuitemTGRemoteDisplay},
                                $gui{menuitemTGKeyboard},
                                $gui{toolbuttonStart},
                                $gui{toolbuttonStop},
                                $gui{toolbuttonSettings},
                                $gui{toolbuttonRemoteDisplay},
                                $gui{toolbuttonReset},
                                $gui{buttonShowDetails},
                                $gui{buttonShowSnapshots},
                                $gui{buttonSaveDescription}],
                stor_unsel  => [$gui{buttonEditStorAddAttach},
                                $gui{buttonEditStorRemoveAttach},
                                $gui{buttonEditStorRemoveCtr},
                                $gui{comboboxEditStorCtrType}],
                ctr_sel_on  => [$gui{buttonEditStorAddAttach},
                                $gui{buttonEditStorRemoveCtr},
                                $gui{comboboxEditStorCtrType}],
                ctr_sel_off => [$gui{buttonEditStorAddAttach},
                                $gui{buttonEditStorRemoveCtr},
                                $gui{comboboxEditStorCtrType}]);

# Simple GUI subs
sub sensitive_on() { $_->set_sensitive(1) foreach (@_); }
sub sensitive_off() { $_->set_sensitive(0) foreach (@_); }
sub hide_on() { $_->hide foreach (@_); }
sub hide_off() { $_->show foreach (@_); }
sub clear_liststore() { $_->clear() foreach (@_); }

# Appends a new entry to a liststore
sub append_liststore() {
    my $liststore = shift(@_);
    my $iter = $liststore->append;
    $liststore->set($iter, @_);
    return $iter;
}

# Ghosts window and sets busy pointer
sub busy_window() {
    &sensitive_off(@_);
    foreach (@_) {
        eval{ $_->window->set_cursor(Gtk2::Gdk::Cursor->new('watch')); };
        Gtk2->main_iteration() while Gtk2->events_pending();
    }
}

# Unghosts window and sets normal pointer
sub unbusy_window() {
    &sensitive_on(@_);
    foreach (@_) {
        eval{ $_->window->set_cursor(Gtk2::Gdk::Cursor->new('left-ptr')); };
    }
}

# Sets the status message for every status bar
sub set_status_msg() {
    my ($msg) = @_;
    my @bars = ($gui{statusbarMain}, $gui{statusbarVMM});
    foreach my $bar (@bars) {
        $bar->pop($bar->get_context_id('General')); # Remove any old msg, harmless if there isn't one
        $bar->push($bar->get_context_id('General'), $msg);
    }
    Gtk2->main_iteration() while Gtk2->events_pending();
}

# Returns selected entry from a TreeView or Combobox. If no column is defined an array of all
# columns is returned, otherwise just that column is returned. wantarray() wasn't suitable here
sub get_selected() {
    my ($widget, $col) = @_;
    my ($model, $iter);

    if ($widget =~ m/^Gtk2::TreeView/) { ($model, $iter) = $widget->get_selection->get_selected;
    }
    else {
        $iter = $widget->get_active_iter();
        $model = $widget->get_model();
    }

    if (defined($col)) { return $model->get($iter,$col); }
    else { return $model->get($iter); }
}

sub show_error_dialog() {
    my ($msg1, $msg2) = @_;
    $gui{messagedialogError}->set_markup("<big><b>$msg1</b></big>");
    if ($msg2) { $gui{messagedialogError}->format_secondary_markup($msg2); }
    $gui{messagedialogError}->run;
    $gui{messagedialogError}->hide;
}

sub show_warn_dialog() {
    my ($msg1, $msg2) = @_;
    $gui{messagedialogWarning}->set_markup("<big><b>$msg1</b></big>");
    if ($msg2) { $gui{messagedialogWarning}->format_secondary_markup($msg2); }
    $gui{messagedialogWarning}->run;
    $gui{messagedialogWarning}->hide;
}

1;
